// Copyright 2010 Google Inc.
// All Rights Reserved.

/**
 * @fileoverview A tween for animating the repositioning of a feed.
 *
 * @author geer@google.com (Arthur van der Geer)
 */

goog.provide('talk.media.c3d.FeedAreaTween');

goog.require('talk.media.c3d.easing');
goog.require('talk.media.c3d.FixedAreaFeed');
goog.require('talk.media.c3d.Tween');


/**
 * A tween that adjusts the size and position of a FixedAreaFeed.
 *
 * @param {talk.media.c3d.FixedAreaFeed} feed The feed to move.
 * @param {number} x The final x position.
 * @param {number} y The final y position.
 * @param {number} width The final width.
 * @param {number} height The final height.
 * @param {number} duration The duration of the tween.
 * @param {talk.media.c3d.EasingMethod} easeMethod The easing method.
 * @param {Function} opt_doneFn A function to be called when tween finishes.
 * @constructor
 * @extends talk.media.c3d.Tween
 */
talk.media.c3d.FeedAreaTween = function(feed, x, y, width, height, duration,
    easeMethod, opt_doneFn) {
  talk.media.c3d.Tween.call(this, duration, opt_doneFn);

  /**
   * The feed to tween.
   *
   * @type {talk.media.c3d.FixedAreaFeed}
   * @private
   */
  this.feed_ = feed;


  /**
   * The initial x position.
   *
   * @type {number}
   * @private
   */
  this.startX_ = feed.getX();


  /**
   * The initial y position.
   *
   * @type {number}
   * @private
   */
  this.startY_ = feed.getY();


  /**
   * The initial width.
   *
   * @type {number}
   * @private
   */
  this.startWidth_ = feed.getWidth();


  /**
   * The initial height.
   *
   * @type {number}
   * @private
   */
  this.startHeight_ = feed.getHeight();


  /**
   * The final x position.
   *
   * @type {number}
   * @private
   */
  this.endX_ = x;


  /**
   * The final y position.
   *
   * @type {number}
   * @private
   */
  this.endY_ = y;


  /**
   * The final width.
   *
   * @type {number}
   * @private
   */
  this.endWidth_ = width;


  /**
   * The final height.
   *
   * @type {number}
   * @private
   */
  this.endHeight_ = height;


  /**
   * The easing method.
   *
   * @type talk.media.c3d.EasingMethod
   * @private
   */
  this.easeMethod_ = easeMethod;

};
goog.inherits(talk.media.c3d.FeedAreaTween, talk.media.c3d.Tween);


/**
 * {@inheritDoc}
 */
talk.media.c3d.FeedAreaTween.prototype.doTween_ = function(fraction) {
  this.feed_.setArea(
      this.easeMethod_.doEase(this.startX_, this.endX_, fraction),
      this.easeMethod_.doEase(this.startY_, this.endY_, fraction),
      this.easeMethod_.doEase(this.startWidth_, this.endWidth_, fraction),
      this.easeMethod_.doEase(this.startHeight_, this.endHeight_, fraction));
};
