// Copyright 2009 Google Inc.
// All Rights Reserved.

/**
 * @fileoverview Handles the layout of multiple video views in N:N calls in a
 *     horizontal strip.
 *
 * @author mikaeld@google.com (Mikael Drugge)
 * @author geer@google.com (Arthur van der Geer)
 */

goog.provide('talk.media.c3d.StripScene');

goog.require('talk.media.c3d.FeedAreaTween');
goog.require('talk.media.c3d.LabeledFeed');
goog.require('talk.media.c3d.SceneBase');


/**
 * Creates the StripScene.
 *
 * @param {talk.media.c3d.O3dBundle} o3dBundle The bundle of global resources
 *     for this O3D instance.
 * @param {boolean} orderFeeds Whether or not to order the video feeds.
 * @extends {talk.media.c3d.SceneBase}
 * @constructor
 */
talk.media.c3d.StripScene = function(o3dBundle, orderFeeds) {
  talk.media.c3d.SceneBase.call(this, o3dBundle);

  /**
   * Whether video feeds should be ordered or not.
   *
   * @type {boolean}
   * @private
   */
  this.orderFeeds_ = orderFeeds;

};
goog.inherits(talk.media.c3d.StripScene, talk.media.c3d.SceneBase);


/**
 * The default number of user feeds to fit on screen initially.
 *
 * @type {number}
 * @const
 * @private
 */
talk.media.c3d.StripScene.USER_FEEDS_ = 5;


/**
 * For each feed we allocate screen real estate with a 16:10 aspect ratio.
 * If the feed is not 16:10 then the sizing logic in FixedAreaFeed will either
 * pillarbox or letterbox it within its available area.
 *
 * @type {number}
 * @const
 */
talk.media.c3d.StripScene.FEED_ASPECT = 1.6;


/**
 * The duration of moving a feed with an animation.
 *
 * @type {number}
 * @const
 * @private
 */
talk.media.c3d.StripScene.FEED_MOVE_DURATION_ = 1,


/**
 * @override
 * @protected
 */
talk.media.c3d.StripScene.prototype.createFeed = function(endpoint) {
  return new talk.media.c3d.LabeledFeed(
      this.o3dBundle, endpoint, 0.2, 0, 0, 0, 0, this);
};


/**
 * @override
 * @protected
 */
talk.media.c3d.StripScene.prototype.updatePositions = function(opt_animate) {
  var count = this.getFeedCount();

  var scale = 0.92;  // For spacing in between the video feeds.
  var spacing = (1 - scale) / 2;

  if (this.width_ / this.height_ >= talk.media.c3d.StripScene.FEED_ASPECT) {
    // Arrange feeds horizontally.
    var width = this.width_ /
        Math.max(talk.media.c3d.StripScene.USER_FEEDS_, count);
    var height = width / talk.media.c3d.StripScene.FEED_ASPECT;
    if (height > this.height_) {
      // If we make our feeds take up the available horizontal space then
      // they'll be too high. Must instead limit to the available vertical
      // space and leave some horizontal space unused.
      height = this.height_;
      width = height * talk.media.c3d.StripScene.FEED_ASPECT;
    }
    // Loop through all video feeds and (re)position them centered.
    var streams = this.getStreamIds();
    var middle = this.x_ + this.width_ / 2 - streams.length * width / 2;
    for (var i = 0; i < streams.length; ++i) {
      var feed = /** @type {talk.media.c3d.FixedAreaFeed} */
          (this.getFeed(streams[i]));
      this.moveFeed_(feed,
          middle + (i * width) + spacing * width,
          this.y_ + spacing * height,
          width * scale,
          height * scale,
          opt_animate);
    }
  } else {
    // Vertical strip.
    var height = this.height_ /
        Math.max(talk.media.c3d.StripScene.USER_FEEDS_, count);
    var width = height * talk.media.c3d.StripScene.FEED_ASPECT;
    if (width > this.width_) {
      // If we make our feeds take up the available vertical space then they'll
      // be too wide. Must instead limit to the available horizontal space and
      // leave some vertical space unused.
      width = this.width_;
      height = width / talk.media.c3d.StripScene.FEED_ASPECT;
    }
    // Loop through all video feeds and (re)position them top aligned.
    var streams = this.getStreamIds();
    for (var i = 0; i < streams.length; ++i) {
      var feed = /** @type {talk.media.c3d.FixedAreaFeed} */
          (this.getFeed(streams[i]));
      this.moveFeed_(feed,
          this.x_ + spacing * width,
          this.y_ + (i * height) + spacing * height,
          width * scale,
          height * scale,
          opt_animate);
    }
  }
};


/**
 * Return the height of this strip for a given available width.
 *
 * @param {number} width The available horizontal space.
 * @return {number} The matching height.
 */
talk.media.c3d.StripScene.prototype.getHeightForWidth = function(width) {
  var feedWidth = width /
      Math.max(talk.media.c3d.StripScene.USER_FEEDS_, this.getFeedCount());
  return feedWidth / talk.media.c3d.StripScene.FEED_ASPECT;
};


/**
 * Return the width of this strip for a given available height.
 *
 * @param {number} height The available vertical space.
 * @return {number} The matching width.
 */
talk.media.c3d.StripScene.prototype.getWidthForHeight = function(height) {
  var feedHeight = height /
      Math.max(talk.media.c3d.StripScene.USER_FEEDS_, this.getFeedCount());
  return feedHeight * talk.media.c3d.StripScene.FEED_ASPECT;
};


/**
 * @override
 */
talk.media.c3d.StripScene.prototype.getStreamIds = function() {
  var ids = goog.base(this, 'getStreamIds');

  if (this.orderFeeds_) {
    // TODO(geer): Sort on name instead of jid as soon as name is shown in UI.
    ids.sort(goog.bind(function(a, b) {
        var jidA = this.getFeed(a).getEndpoint().getParticipant().getJid();
        var jidB = this.getFeed(b).getEndpoint().getParticipant().getJid();
        if (jidA < jidB) {
          return -1;
        } else if (jidB < jidA) {
          return 1;
        }
        return 0;
    }, this));
  }
  return ids;
};


/**
 * Moves and sizes a feed, optionally animated.
 *
 * @param {talk.media.c3d.FixedAreaFeed} feed The feed to move.
 * @param {number} x The final x position.
 * @param {number} y The final y position.
 * @param {number} width The final width.
 * @param {number} height The final height.
 * @param {boolean} opt_animate Whether to animate the move.
 * @private
 */
talk.media.c3d.StripScene.prototype.moveFeed_ =
    function(feed, x, y, width, height, opt_animate) {
  // TODO(geer): Make animation default or an independent experiment.
  if (this.orderFeeds_ && feed.feedObject && opt_animate) {
    var tween = new talk.media.c3d.FeedAreaTween(feed, x, y, width, height,
        talk.media.c3d.StripScene.FEED_MOVE_DURATION_,
        talk.media.c3d.STRONG_EASE_IN_OUT,
        null);
    this.o3dBundle.c3dEngine.registerRenderCallback(tween);
  } else {
    feed.setArea(x, y, width, height);
  }
};
