// Copyright 2008 Google Inc.
// All Rights Reserved.

/**
 * @fileoverview Calling related callonstants factored out to reduce cyclic
 * dependency issues.
 *
 * @author Scott Shawcroft (shawcroft@google.com)
 */

goog.provide('talk.media.CallableState');
goog.provide('talk.media.CallType');
goog.provide('talk.media.IdleSignal');
goog.provide('talk.media.SwitchMediaUiEvent');
goog.provide('talk.media.TalkPluginEvent');
goog.provide('talk.media.TalkPluginErrorCodes');
goog.provide('talk.media.UiEvents');
goog.provide('talk.media.UiMessageTypes')
goog.provide('talk.media.UiMessages');
goog.provide('talk.media.UiMode');

goog.require('goog.events');

/**
 * Enum for identifying call types.
 * Matches the google/talk/session/CallSession.as call types
 *
 * @enum {number}
 */
talk.media.CallType = {
  NONE : 0,

  VOICE : 1,

  VOICEMAIL : 2,

  VIDEO : 3,

  TEST : 4
};

/**
 * Enum for identifying native idle signal.
 *
 * @enum {number}
 */
talk.media.IdleSignal = {
  /** Plugin does not support idle detection. */
  NONE : 0,

  /** Not idle. */
  ACTIVE : 1,

  /** Idle. */
  IDLE : 2
};


/**
 * Events fired by the Talk Plugin.
 *
 * @enum {string}
 */
talk.media.TalkPluginEvent = {
  /**
   * Event indiciating the version of magicflute has been determined or
   * changed.
   */
  VERSION : 'mf',

  /** Event indicating an error was generated by the Google Talk Plugin. */
  ERROR : 'fe',

  /**
    * Event indicating that the Google Talk Plugin is closing an active session.
    * Causes include a laptop going in to standby state.
    */
  END_CALL : 'end',

  /**
   * Event indicating that we received a message from the Google Talk Plugin.
   */
  MESSAGE : 'f-msg',

  /**
   * Event indicating that we are sending a message to the Google Talk Plugin.
   */
  OUTGOING_MESSAGE : 'of-msg',

  /**
   * Video stream off message, sent to stop rendering of specific video feeds.
   */
  STREAM_OFF : 'streamoff',

  /**
   * Event to pass available devices and device preferences from Google Talk
   * Plugin.
   */
  GET_DEVICE_PREFS : 'getdevicestate',

  /** Event to pass device preferences to Google Talk Plugin. */
  SET_DEVICE_PREFS : 'setdevicestate',

  /**
   * Message indicating that magicflute is running and ready.
   */
  READY : 'f-ready',

  /**
   * Message indicating that the state of flute has changed. For example when
   * call type, call state, or idle state changes.
   */
  FLUTESTATE_CHANGED : 'flutestate',

  /** Message indicating that magicflute has exited (as if from a crash). */
  DEAD : 'f-dead',

  /**
   * Message indicating progress of blowser plugin to magicflute connection
   * establishment.
   */
  CONNECTING : 'f-connect',

  /** Event to pass audio information from the Google Talk Plugin. */
  AUDIO_INFO : 'jai',

  /** Indicates that the log upload was completed. */
  UPLOAD_LOG_RESPONSE: 'uploadlogresponse',

  /**
   * Indicates the plugin channel has collected state about its connection
   * establishment process that it wants interpreted and recorded.
   */
  CONNECT_IMPRESSION : 'log-connect',

  /**
   * Updates us about the connection state of the call, informing us about
   * whether any or all of the media transports can be written to.
   */
  TRANSPORT_STATE : 'trstate',

  /** Contains a list of currently visible windows. */
  SHARE_WINDOWS : 'share-windows'
};

/**
 * Codes sent up by the talk plugin in the flute error ('fe') message.
 *
 * @enum {number}
 */
talk.media.TalkPluginErrorCodes = {
  CONNECTIVITY_LOST : 1
};

/**
 * Contains events pertaining to the media user interface.
 *
 * @enum {string}
 */
talk.media.UiEvents = {
  /**
   * Indicates that the user has requested the media rendering UI be changed,
   * but that the stream should not be terminated.
   *
   * In other words, this is used when switching between renderering modes.
   */
  SWITCH_MEDIA_UI :  goog.events.getUniqueId('close_media_ui')
};

/**
 * An event for switching between rendering modes.
 *
 * @param {talk.media.UiMode} desiredMode The UI mode the user wants.
 * @constructor
 * @extends {goog.events.Event}
 */
talk.media.SwitchMediaUiEvent = function(desiredMode) {
  goog.events.Event.call(this, talk.media.UiEvents.SWITCH_MEDIA_UI);

  /**
   * @type {talk.media.UiMode}
   */
  this.desiredMode = desiredMode;
};
goog.inherits(talk.media.SwitchMediaUiEvent, goog.events.Event);

/**
 * Types of messages that we need to pass down into the renderer.
 *
 * @enum {string}
 */
talk.media.UiMessageTypes = {

  /** Messages from the Google Talk Plugin. */
  TALK_PLUGIN : 'plugin-data',

  /** Jingle singaling messages. */
  JINGLE : 'connect-data'
};

/**
 * Types of Jingle/Flute messages that the UI cares about.
 *
 * @type {Object}
 */
talk.media.UiMessages = {
  'streamon' : 1,
  'streamoff' : 1,
  'streamready' : 1,
  'streamfailed' : 1,
  'jrng' : 1,
  'ja' : 1
};

/**
 * Different modes for the media UI.
 *
 * @enum {string}
 */
talk.media.UiMode = {
  /**
   * A large mode, intended to take over the underlying property hosting the
   * media session.
   */
  BIG : 'b',

  /**
   * A smaller mode, intended for multitasking with the underlying property
   * while in a media session.
   */
  NORMAL : 'n'
};

/**
 * The index of the audio input level in a jingle audio info message.
 * @type {number}
 */
talk.media.AUDIO_INFO_INPUT_LEVEL_PARAM = 3;

/**
 * The index of the array of speaker info in a jingle audio info message.
 * @type {number}
 */
talk.media.AUDIO_INFO_SPEAKERS_PARAM = 4;

/**
 * The index of the output level of the local user within the speaker info
 * array.
 * @type {number}
 */
talk.media.AUDIO_INFO_SPEAKERS_SELF = 0;

/**
 * The index of the user volumes within the speaker info array.
 * @type {number}
 */
talk.media.AUDIO_INFO_SPEAKERS_VOLUMES = 1;


/**
 * What calling action can be taken.
 * @enum {string}
 */
talk.media.CallableState = {
  /**
   * Can make a call.
   */
  CALLABLE: 'c',

  /**
   * Can install the plugin.
   */
  INSTALLABLE: 'p',

  /**
   * Can send an invite to the contact to add the plugin.
   */
  INVITABLE: 'i',

  /**
   * Can't make a call because the other person is offline.
   */
  DISABLED_OFFLINE: 'off',

  /**
   * Can't make a call because the other person is blocked.
   */
  DISABLED_BLOCKED: 'bl',

  /**
   * Can't start a call because already in call.
   */
  DISABLED_IN_CALL: 'ic',

  /**
   * Can't start a call because in a muc and not in N:N experiment
   */
  DISABLED_MUC: 'mu',

  /**
   * Can't start a call or invite because contact is using an unsupported OS.
   */
  DISABLED_UNINVITABLE: 'os',

  /**
   * Can't start a call or invite because contact is from AIM.
   */
  DISABLED_AIM: 'aim',

  /**
   * Can't start a call because the Voice setup hasn't been completed.
   */
  PHONE_NOT_SETUP: 'pns',

  /**
   * Voice/video is disabled by domain policy.
   */
  DISABLED_POLICY: 'pol',

  /**
   * This is an unsupported configuration, the chat is in an unexpected
   * state (call manager not present) or we can't start a call for an
   * unknown reason. In this state, the icon shouldn't be visible.
   */
  HIDDEN_UNAVAILABLE: 'un'
};


/**
 * The number of MS the plugin will wait to time out a call that has not
 * connected.
 * @type {number}
 */
talk.media.CALL_TIMEOUT_MS = 90 * 1000;
