// Copyright 2010 Google Inc.
// All Rights Reserved.

/**
 * @fileoverview Crypto utility functions.
 *
 * @author mikaeld@google.com (Mikael Drugge)
 */

goog.provide('talk.media.Crypto');

goog.require('goog.debug.Logger');

/**
 * @constructor
 */
talk.media.Crypto = function() {
};


/**
 * The Crypto singleton instance.
 *
 * @type {talk.media.Crypto}
 * @private
 */
talk.media.Crypto.INSTANCE_;


/**
 * Returns the Crypto instance.
 *
 * @return {!talk.media.Crypto} The access control.
 */
talk.media.Crypto.getInstance = function() {
  if (!goog.isDefAndNotNull(talk.media.Crypto.INSTANCE_)) {
    talk.media.Crypto.INSTANCE_ = new talk.media.Crypto();
  }
  return talk.media.Crypto.INSTANCE_;
};


/**
 * Logger for debugging.
 *
 * @type {!goog.debug.Logger}
 * @private
 */
talk.media.Crypto.prototype.logger_ =
    goog.debug.Logger.getLogger('talk.media.Crypto');


/**
 * Returns whether the given crypto json object has encryption enabled.
 *
 * @param {Object} json The json crypto object.
 * @return {boolean} Whether encryption is enabled.
 */
talk.media.Crypto.prototype.hasEncryption = function(json) {
  if (json) {
    var suites = this.getCipherSuites_(json);

    for (var i = 0; i < suites.length; i++) {
      if (this.isCipherSuiteSecure_(suites[i])) {
        return true;
      }
    }
  }

  return false;
};


/**
 * Returns an array of crypto suites from a json crypto object.
 *
 * @param {Object} json The json crypto object.
 * @return {Array.<string>} The crypto suites.
 * @private
 */
talk.media.Crypto.prototype.getCipherSuites_ = function(json) {
  var obj = json['crypto'];
  if (!obj) {
    return [];
  }

  if (obj instanceof Array) {
    var list = [];
    for (var i = 0; i < json['crypto'].length; i++) {
      list.push(obj[i]['cs']);
    }
    return list;
  } else {
    return [obj['cs']];
  }
};


/**
 * Returns whether the given crypto suite is deemed secure.
 *
 * @param {string} suite The crypto suite.
 * @return {boolean} Whether the given crypto suite is deemed secure.
 * @private
 */
talk.media.Crypto.prototype.isCipherSuiteSecure_ = function(suite) {
  return !!(suite && suite != 'NULL');
};
