// Copyright 2007 Google Inc.
// All Rights Reserved.

/**
 * @fileoverview Defines the talk.media.FlashPlayer class.
 * FlashPlayer is a javascript wrapper around a basic, closable SWF file
 * Note that the content of {@code flashDiv} will be overwritten with the SWF.
 * To handle messages coming in from the SWF the subclass should extend
 * handleMessage. To perform any cleanup necessary when closing,
 * subclasses can extend onClose. Subclasses can call sendToPlayer
 * to send messages to the SWF.
 *
 * <p> This class expects to interact with SWFs that use
 * flash/actionscript/com/google/talk/BrowserInterface.as
 * to communicate.
 *
 * @author jessan@google.com (Jessan Hutchison-Quillian)
 */

goog.provide('talk.media.FlashPlayer');

goog.require('goog.events.EventTarget');
goog.require('talk.media.FlashObject');

/**
 * A basic Flash UI object that understands how to add and remove
 * itself from the DOM.
 *
 * @param {string} name A name identifying the SWF
 * @param {string} url The URL to the SWF
 * @param {string} opt_flashVars Parameters to be passed to the SWF.
 *     The string should be in URL parameter format: var1=A&var2=B...
 * @constructor
 * @extends {goog.events.EventTarget}
 */
talk.media.FlashPlayer = function(name, url, opt_flashVars) {
  this.playerSwf_ = new talk.media.FlashObject(name,
      talk.media.FlashPlayer.PREFIX);
  this.playerSwf_.setRecv(goog.bind(this.onRecvFromPlayer_, this));
  this.url_ = url;

  /**
   * Parameters to be passed to the SWF.
   * @type {string|undefined}
   * @private
   */
  this.flashVars_ = opt_flashVars;
};
goog.inherits(talk.media.FlashPlayer, goog.events.EventTarget);


/**
 * Logger
 * @type goog.debug.Logger
 * @private
 */
talk.media.FlashPlayer.logger_ =
    goog.debug.Logger.getLogger('talk.media.FlashPlayer');


/**
 * @type talk.media.FlashObject
 * @private
 */
talk.media.FlashPlayer.prototype.playerSwf_;


/**
 * The index of the message type in a message from Flash.
 * @type number
 */
talk.media.FlashPlayer.MESSAGE_TYPE_INDEX = 0;


/**
 * The function to use for writing the embed or object tag to the inner HTML of
 * the container of the SWF. If this function is not defined in a
 * <script src=".."> tag then the SWF will be shown with a "click to activate"
 * tooltip in internet explorer.
 *
 * @type Function
 */
talk.media.FlashPlayer.writeInnerHtmlFn;


/**
 * Events generated by the FlashPlayer.
 *
 * @enum {string}
 */
talk.media.FlashPlayer.Events = {
  /**
   * An event indicating that the user has requested the player be closed.
   */
  CLOSE : 'player_close',

  /**
   * An event indicating that the user has clicked the SWF.
   */
  CLICKED : 'CLICKED'
};


/**
 * Enum for messages received from the SWF.
 *
 * @enum {string}
 */
talk.media.FlashPlayer.RecvMsg = {
  /**
   * Request to close the player
   */
  CLOSE_MSG : 'CLOSE',

  /**
   * A debug message from flash
   */
  TRACE_MSG : 'trace',

  /**
   * Message indicating the player was clicked
   */
  CLICK_MSG : 'CLICK'
};


/**
 * The prefix identifying messages from this kind of object.
 *
 * @type {string}
 */
talk.media.FlashPlayer.PREFIX = 'player';


/**
 * Returns the default wmode value, window.
 *
 * @return {talk.media.FlashObject.Wmodes} The Wmode to use.
 */
talk.media.FlashPlayer.prototype.getWmode = function() {
  return talk.media.FlashObject.Wmodes.WINDOW;
};


/**
 * Forwards the message to the media player.
 * Protected: should only be used by subclasses.
 *
 * @param {string} msg the kind of message to send to the player
 * @param {Array|string|Object} args the arguments to be sent with the message
 */
talk.media.FlashPlayer.prototype.sendToPlayer = function (msg, args) {
  /** @type {Array} */
  var argsArray;
  if (typeof args == 'string') {
    argsArray = /** @type {Array} */ (goog.json.parse(args));
  } else {
    argsArray = /** @type {Array} */ (args);
  }
  this.playerSwf_.send(talk.media.FlashPlayer.PREFIX + msg, argsArray);
};


/**
 * Handles a message from the flash player.
 *
 * @param {string} src unused
 * @param {string} dest unused
 * @param {Array.<string>} msg Message array, contents depend on type of msg
 * @private
 */
talk.media.FlashPlayer.prototype.onRecvFromPlayer_ = function(src, dest, msg) {
  var type = msg[talk.media.FlashPlayer.MESSAGE_TYPE_INDEX]
  switch (type) {
    case talk.media.FlashPlayer.RecvMsg.CLOSE_MSG:
      this.onClose();
      break;
    case talk.media.FlashPlayer.RecvMsg.TRACE_MSG:
      talk.media.FlashPlayer.logger_.info("Flash Trace:" + msg[1]);
      break;
    case talk.media.FlashPlayer.RecvMsg.CLICK_MSG:
      this.dispatchEvent(talk.media.FlashPlayer.Events.CLICKED);
      break;
    default:
      if (!this.handleMessage(type, msg)) {
         this.handleUnknownMessage_(msg);
      }
  }
};


/**
 * Called when a close button within the player is clicked.
 */
talk.media.FlashPlayer.prototype.onClose = function() {
  this.dispatchEvent(talk.media.FlashPlayer.Events.CLOSE);
};


/**
 * Provides an opportunity for subclasses to handle an incoming message
 * Protected: should be overridden by subclasses
 *
 * @param {string} type The type of the message.
 * @param {Array.<string>} msg Message array, contents depend on message type
 * @return {boolean} Whether or not the message is handled.
 */
talk.media.FlashPlayer.prototype.handleMessage = function(type, msg) {
  return false;
};


/**
 * Handles a message from the flash player whose type we don't know.
 *
 * @param {Array.<string>} msg Message array, contents depend on type of msg
 * @private
 */
talk.media.FlashPlayer.prototype.handleUnknownMessage_ = function(msg) {
  talk.media.FlashPlayer.logger_.info("Received unknown flash message: " + msg);
};


/**
 * Removes the HTML from the player SWF from the DOM so the browser unloads it.
 */
talk.media.FlashPlayer.prototype.removePlayer = function() {
  if (this.playerSwf_) {
    this.playerSwf_.removeHTML();
  }
};


/**
 * Resets the SWF's state. The SWF will behave as if it has not been loaded.
 */
talk.media.FlashPlayer.prototype.resetState = function() {
  this.playerSwf_.resetState();
};


/**
 * Adds itself to the DOM by writing the embed/object tag into the Element,
 * overwriting its previous contents.
 *
 * @param {string|Element} swfContainer An Element into which we insert the SWF.
 */
talk.media.FlashPlayer.prototype.setContainer = function(swfContainer) {
  this.playerSwf_.writeHTML(swfContainer, this.url_, this.flashVars_, true,
      this.getWmode(), undefined, talk.media.FlashPlayer.writeInnerHtmlFn);
};
