// Copyright 2010 Google Inc. All Rights Reserved.

/**
 * @fileoverview An object that represents a participant in a voice or video
 * meeting.
 *
 * @author geer@google.com (Arthur van der Geer)
 */

goog.provide('talk.media.Participant');


/**
 * Construct a meeting participant.
 *
 * @param {string} jid The bare JID of the participant.
 * @param {string} name The name to display.
 * @param {string} email The email address.
 * @param {string} avatar The url of the avatar image.
 * @param {boolean} isLocal Indication if this participant is the local user.
 * @constructor
 */
talk.media.Participant = function(jid, name, email, avatar, isLocal) {

  /**
   * The bare JID of the participant.
   *
   * @type {string}
   * @private
   */
  this.jid_ = jid;

  /**
   * The name to display.
   *
   * @type {string}
   * @private
   */
  this.name_ = name;

  /**
   * The email address.
   *
   * @type {string}
   * @private
   */
  this.email_ = email;

  /**
   * The url of the avatar image.
   *
   * @type {string}
   * @private
   */
  this.avatar_ = avatar;

  /**
   * Indication if this participant is the local user.
   *
   * @type {boolean}
   * @private
   */
  this.isLocal_ = isLocal;
};


/**
 * The SSRC of the audio stream.
 *
 * @type {?string}
 * @private
 */
talk.media.Participant.prototype.audioSsrc_;


/**
 * The SSRC of the video stream.
 *
 * @type {?string}
 * @private
 */
talk.media.Participant.prototype.videoSsrc_;


/**
 * @return {string} The bare JID of the participant.
 */
talk.media.Participant.prototype.getJid = function() {
  return this.jid_;
};


/**
 * @return {string} The name to display.
 */
talk.media.Participant.prototype.getName = function() {
  return this.name_;
};


/**
 * Set the name to display.
 *
 * @param {string} name The name to display.
 */
talk.media.Participant.prototype.setName = function(name) {
  this.name_ = name;
};


/**
 * @return {string} The email address.
 */
talk.media.Participant.prototype.getEmail = function() {
  return this.email_;
};


/**
 * Set the email address.
 *
 * @param {string} email The email address.
 */
talk.media.Participant.prototype.setEmail = function(email) {
  this.email_ = email;
};


/**
 * @return {string} The url of the avatar image.
 */
talk.media.Participant.prototype.getAvatar = function() {
  return this.avatar_;
};


/**
 * Set the url of the avatar image.
 *
 * @param {string} avatar The url of the avatar image.
 */
talk.media.Participant.prototype.setAvatar = function(avatar) {
  this.avatar_ = avatar;
};


/**
 * @return {boolean} Indication if this participant is the local user or not.
 */
talk.media.Participant.prototype.isLocal = function() {
  return this.isLocal_;
};


/**
 * @return {?string} The SSRC of the audio stream.
 */
talk.media.Participant.prototype.getAudioSSRC = function() {
  return this.audioSsrc_;
};


/**
 * Set the SSRC of the audio stream.
 *
 * @param {?string} ssrc The SSRC of the audio stream.
 */
talk.media.Participant.prototype.setAudioSSRC = function(ssrc) {
  this.audioSsrc_ = ssrc;
};


/**
 * @return {?string} The SSRC of the video stream.
 */
talk.media.Participant.prototype.getVideoSSRC = function() {
  return this.videoSsrc_;
};


/**
 * Set the SSRC of the video stream.
 *
 * @param {?string} ssrc The SSRC of the video stream.
 */
talk.media.Participant.prototype.setVideoSSRC = function(ssrc) {
  this.videoSsrc_ = ssrc;
};
