// Copyright 2010 Google Inc. All Rights Reserved.

/**
 * @fileoverview Controls the resolution and framerate of video feeds based on
 * the size of the views displaying them.
 *
 * @author geer@google.com (Arthur van der Geer)
 */

goog.provide('talk.media.ViewQualityControl');

goog.require('goog.Timer');
goog.require('goog.debug.Logger');
goog.require('goog.structs.Map');
goog.require('talk.media.CallManager');


/**
 * Creates an object that controls the resolution of video feeds based on the
 * size of the views displaying them.
 *
 * @param {talk.media.CallManager} callManager The object handling communication
 *     with the server.
 * @constructor
 * @extends {goog.Disposable}
 */
talk.media.ViewQualityControl = function(callManager) {
  goog.Disposable.call(this);

  /**
   * The call manager handles communication with the server.
   *
   * @type {talk.media.CallManager}
   * @private
   */
  this.callManager_ = callManager;

};
goog.inherits(talk.media.ViewQualityControl, goog.Disposable);


/**
 * The amount of time in ms to wait between sending view requests to the server.
 *
 * @type {number}
 * @const
 * @private
 */
talk.media.ViewQualityControl.WAIT_BETWEEN_SEND_MS_ = 500;


/**
 * Default number of video frames per second to request from the server.
 *
 * @type {number}
 * @const
 * @private
 */
talk.media.ViewQualityControl.DEFAULT_VIDEO_FRAMERATE_ = 30;


/**
 * Logger for debugging.
 *
 * @type {goog.debug.Logger}
 * @private
 */
talk.media.ViewQualityControl.logger_ =
    goog.debug.Logger.getLogger('talk.media.ViewQualityControl');


/**
 * The last unsent view request message, representing the last valid state.
 *
 * @type {Array}
 * @private
 */
talk.media.ViewQualityControl.prototype.outstandingMsg_;


/**
 * The ID of the timer that reenables sending after a send.
 *
 * @type {?number}
 * @private
 */
talk.media.ViewQualityControl.prototype.recentSendTimer_;


/**
 * Changes the resolution and framerate of all video feeds to match the sizes
 * of the views.
 *
 * @param {Array.<Array>} views The views and their sizes. The array elements
 *     for each view are [{string} SSRC, {number} width, {number} height].
 */
talk.media.ViewQualityControl.prototype.adjustQualityForViews =
    function(views) {
  var call = this.callManager_.getActiveCall();
  if (!goog.isDefAndNotNull(call) || !goog.isDefAndNotNull(call.ssrcMap)) {
    return;
  }

  // Take the largest view of each ssrc.
  var streamQuality = new goog.structs.Map();
  for (var i = 0; i < views.length; ++i) {
    var v = views[i];
    var other = streamQuality.get(v[0]);
    if (other == undefined || v[2] > other[2]) {
      streamQuality.set(v[0], [v[0], v[1], v[2]]);
    }
  }
  this.requestQuality_(streamQuality);
};


/**
 * Sends a 'view' message to the conference manager in the backend to request
 * the desired quality for each video stream.
 *
 * @param {goog.structs.Map} streamQuality The desired quality of each stream.
 * @private
 */
talk.media.ViewQualityControl.prototype.requestQuality_ =
    function(streamQuality) {
  var call = this.callManager_.getActiveCall();

  var requests = [];
  var nicks = call.ssrcMap.getKeys();
  nicks.sort();  // For ease of debugging.
  for (var i = 0; i < nicks.length; i++) {
    var entry = call.ssrcMap.get(nicks[i]);
    if (entry.videoSsrc) {
      var resolution = streamQuality.get(entry.videoSsrc);
      // Append the video quality for this nick to the view message.
      if (resolution != undefined) {
        requests.push(['selected', nicks[i], entry.videoSsrc,
            Math.round(resolution[1]), Math.round(resolution[2]),
            talk.media.ViewQualityControl.DEFAULT_VIDEO_FRAMERATE_]);
      } else {
        talk.media.ViewQualityControl.logger_.severe(
            'No view requested for video SSRC: "' + entry.videoSsrc + '".');
      }
    }
  }
  var remoteJid = call.remoteJids[0]
  var msg = ['view', remoteJid, call.sessionId, requests];

  this.sendOrScheduleViewRequest_(msg);
};


/**
 * Sends a message directly if enough time elapsed since the last send.
 * Otherwise keep this message, representing the last valid state, and schedule
 * to send it at the first occasion.
 *
 * @param {Array} msg The message to be sent.
 * @private
 */
talk.media.ViewQualityControl.prototype.sendOrScheduleViewRequest_ =
    function(msg) {
  if (this.recentSendTimer_) {
    this.outstandingMsg_ = msg;
  } else {
    talk.media.ViewQualityControl.logger_.info('Requesting video view: ' +
        goog.json.serialize(msg));
    this.callManager_.sendJingleMsg(msg);
    this.recentSendTimer_ = goog.Timer.callOnce(this.onReadyToSend_,
        talk.media.ViewQualityControl.WAIT_BETWEEN_SEND_MS_, this);
  }
};


/**
 * Enable sending of view requests and send the pending one if needed.
 *
 * @private
 */
talk.media.ViewQualityControl.prototype.onReadyToSend_ = function() {
  this.recentSendTimer_ = null;
  if (this.outstandingMsg_) {
    this.sendOrScheduleViewRequest_(this.outstandingMsg_);
    this.outstandingMsg_ = null;
  }
};


/**
 * @override
 * @protected
 */
talk.media.ViewQualityControl.prototype.disposeInternal = function() {
  talk.media.ViewQualityControl.superClass_.disposeInternal.call(this);
  if (this.recentSendTimer_) {
    goog.Timer.clear(this.recentSendTimer_);
    this.recentSendTimer_ = null;
  }
  this.callManager_ = null;
};
